@@ -5,7 +5,7 @@
 ;; Author: Thanos Apollo <public@thanosapollo.org>
 ;; Keywords: extensions
 ;; URL: https://thanosapollo.org/projects/org-gnosis/
-;; Version: 0.1.0
+;; Version: 0.1.1
 
 ;; Package-Requires: ((emacs "27.2") (emacsql "4.0.0") (compat "29.1.4.2"))
 
@@ -75,6 +75,13 @@
   "TODO files used for the journal entries."
   :type '(repeat string))
 
+(defcustom org-gnosis-todo-keywords org-todo-keywords
+  "TODO Keywords used for parsing `org-gnosis-todo-files'.
+
+All items after the vertical bar \"|\" will be ignored, for
+compatability with `org-todo-keywords'."
+  :type '(repeat string))
+
 (defcustom org-gnosis-completing-read-func #'org-completing-read
   "Function to use for `completing-read'."
   :type 'function)
@@ -179,18 +186,16 @@ to have an ID."
 
 (defun org-gnosis-get-id ()
   "Return id for heading at point."
-  (if (bound-and-true-p org-gnosis-mode)
-      (save-excursion
-	(let ((heading-level (org-current-level))
-	      (id (org-id-get)))
-	  (cond (id id)
-		((and (null id) (= heading-level 1))
-		 (goto-char (point-min))
-		 (org-id-get))
-		(t
-		 (outline-up-heading 1 t)
-		 (org-gnosis-get-id)))))
-    (error "Org Gnosis Mode not enabled")))
+  (save-excursion
+    (let ((heading-level (org-current-level))
+	  (id (org-id-get)))
+      (cond (id id)
+	    ((and (null id) (= heading-level 1))
+	     (goto-char (point-min))
+	     (org-id-get))
+	    (t
+	     (outline-up-heading 1 t)
+	     (org-gnosis-get-id))))))
 
 (defun org-gnosis-collect-id-links ()
   "Collect ID links and current headline ID as (link-id . headline-id) pairs."
@@ -203,9 +208,7 @@ to have an ID."
         (let ((link (match-string-no-properties 0)))
           (when (string-match "id:\\([^]]+\\)" link)
             (let ((target-id (match-string 1 link))
-                  (source-id (save-excursion
-                              (org-back-to-heading-or-point-min t)
-                              (org-id-get))))
+                  (source-id (org-gnosis-get-id)))
               (when (and target-id source-id)
                 (push (cons target-id source-id) links)))))))
     (nreverse links)))
@@ -333,7 +336,8 @@ If JOURNAL is non-nil, update file as a journal entry."
 			      (setq counter (1+ counter)))
 			    (setq title (format "%s (%d)" title counter))
 			    (message "Duplicate title found, renamed to: %s" title)))
-			(org-gnosis--insert-into table `([,id ,filename ,title ,level ,tags]))
+			(org-gnosis--insert-into table `([,id ,filename ,title ,level
+							      ,(prin1-to-string tags)]))
 			;; Insert tags
 			(cl-loop for tag in tags
 				 do
@@ -399,9 +403,11 @@ Delete file contents in database & file."
                    (tags (cadr item))
                    (propertized-tags
 		    (when tags
-                      (concat (propertize "#" 'face 'org-gnosis-face-tags)
-                              (propertize (mapconcat #'identity tags "#")
-					  'face 'org-gnosis-face-tags)))))
+		      (let ((tag-list (if (stringp tags) (read tags) tags)))
+			(when (and tag-list (not (equal tag-list '())))
+                          (concat (propertize "#" 'face 'org-gnosis-face-tags)
+                                  (propertize (mapconcat #'identity tag-list "#")
+					      'face 'org-gnosis-face-tags)))))))
               (if propertized-tags
                   (format "%s  %s" title propertized-tags)
 		title)))
@@ -650,10 +656,13 @@ If file or id are not found, use `org-open-at-point'."
       (org-mode)
       (org-element-map (org-element-parse-buffer) 'headline
         (lambda (headline)
-          (when (string= (org-element-property :todo-keyword headline) "TODO")
+          (when (member (org-element-property :todo-keyword headline)
+			 (cl-loop for keyword in org-gnosis-todo-keywords
+				  until (and (stringp keyword) (string= keyword "|"))
+				  collect keyword))
             (let* ((title (org-element-property :raw-value headline))
-                   (timestamp (org-element-property :raw-value
-                           (org-element-property :scheduled headline))))
+                   (timestamp (org-element-property
+			       :raw-value (org-element-property :scheduled headline))))
               (push `(,title ,timestamp ,file) todos))))))
     (nreverse todos)))
 
@@ -801,7 +810,7 @@ ELEMENT should be the output of `org-element-parse-buffer'."
 			   (not (file-directory-p file))))
 			(directory-files org-gnosis-journal-dir t nil t))
 	   do (org-gnosis-update-file file)))
-
+;; TODO: Rebuil the database upon sync.
 ;;;###autoload
 (defun org-gnosis-db-sync ()
   "Sync `org-gnosis-db'."
